﻿using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System;
//using System.Threading.Tasks;
using System.Windows.Forms;

namespace HumiditySensor_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		HumiditySensor openCh = null;
		formCleaner cleaner;
		HumiditySensor humidity = null;
		private ErrorEventBox errorBox;
		private DataGraphBox humidityGraphBox;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(HumiditySensor ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		private void Form1_Load(object sender, EventArgs e) {
			this.Size = this.MinimumSize;


			errorBox = new ErrorEventBox(this);
			humidityGraphBox = new DataGraphBox(this, "Relative Humidity");
			humidity = openCh ?? openArgs.makeChannel<HumiditySensor>();

			humidity.Attach += humidity_attach;
			humidity.Detach += humidity_detach;
			humidity.Error += humidity_error;
			humidity.HumidityChange += humidity_change;
			humidity.PropertyChange += humidity_propertychange;

			try {
				humidity.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			humidity.Attach -= humidity_attach;
			humidity.Detach -= humidity_detach;
			humidity.Error -= humidity_error;
			humidity.HumidityChange -= humidity_change;
			humidity.PropertyChange -= humidity_propertychange;
			humidity.Close();
		}

		void humidity_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			HumiditySensor attachedDevice = (HumiditySensor)sender;

			humidityGraphBox.SetTitle(DataGraphBox.GenerateTitle("Humidity", (Phidget)attachedDevice));

			try {
				attachedDevice.DataInterval = attachedDevice.MinDataInterval;

				attachedDevice.HumidityChangeTrigger = attachedDevice.MinHumidityChangeTrigger;

				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
				dataIntervalTrk.Value = attachedDevice.DataInterval;

				changeTriggerTrk.Unit = "%";
				changeTriggerTrk.minTxt = attachedDevice.MinHumidityChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinHumidityChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxHumidityChangeTrigger;
				changeTriggerTrk.maxTxt = attachedDevice.MaxHumidityChangeTrigger.ToString();
				changeTriggerTrk.Value = 0;
				changeTriggerTrk.trkText = attachedDevice.HumidityChangeTrigger.ToString();
				changeTriggerTrk.trkText = attachedDevice.HumidityChangeTrigger.ToString("F0");

			} catch (PhidgetException ex) { errorBox.addMessage("Error initializing device: " + ex.Message); }

			settingsBox.Visible = true;
			outputBox.Visible = true;
		}

		void humidity_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			outputBox.Visible = false;
			dataIntervalTrk.TrueValue = -1;
		}

		void humidity_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation)
				humidityGraphBox.addGap();
		}

		private void humidity_propertychange(object sender, PropertyChangeEventArgs e) {

			switch (e.PropertyName) {
			case "DataInterval":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
					dataIntervalTrk.Value = humidity.DataInterval;
					updateGraphDataInterval();
				}
				break;
			case "HumidityChangeTrigger":
				changeTriggerTrk.Value = humidity.HumidityChangeTrigger;
				break;
			case "DataRate":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
					dataIntervalTrk.Value = humidity.DataRate;
					updateGraphDataInterval();
				}
				break;
			}
		}

		void humidity_change(object sender, Phidget22.Events.HumiditySensorHumidityChangeEventArgs e) {
			try {
				humTxt.Text = Math.Round(e.Humidity, 4).ToString() + "%";
				humidityGraphBox.addData(e.Humidity, humidity.DataInterval);
			} catch (PhidgetException ex) { errorBox.addMessage("Error reading humidity: " + ex.Message); }
		}

		private void dataIntervalTrk_Scroll(object sender, EventArgs e) {

		}

		private void updateGraphDataInterval() {
			try {
				if (humidity.HumidityChangeTrigger == 0)
					humidityGraphBox.DataInterval = (1000.0 / humidity.DataRate);
				else
					humidityGraphBox.DataInterval = null;
			} catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}

		private void humidityGraphButton_Click(object sender, EventArgs e) {
			if (!humidityGraphBox.Visible)
				humidityGraphBox.Show();
		}

		private void dataIntervalTrk_KeyOverride(object sender, KeyEventArgs e) {
			e.SuppressKeyPress = true;
			dataIntervalTrk.keyOverride(sender, e);
		}

		private void slider_ValueChangeEvent(object sender, EventArgs e) {
			if (((PhidgetsControlLibrary.CustomScroll)sender).isFixedRange) //datainterval control
			{
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
					try {
						double temp = dataIntervalTrk.Value;
						humidity.DataInterval = (int)temp;
						if (humidity.DataInterval < (int)temp) {
							while (humidity.DataInterval != (int)temp) {
								temp++;
								humidity.DataInterval = (int)temp;
							}
						} else {
							while (humidity.DataInterval != (int)temp) {
								temp--;
								humidity.DataInterval = (int)temp;
							}
						}

						dataIntervalTrk.TrueValue = humidity.DataInterval;
						dataIntervalTrk.Value = humidity.DataInterval;



						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
					try {
						humidity.DataRate = dataIntervalTrk.Value;
						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
				}
			} else //changetrigger control
			  {
				try {
					humidity.HumidityChangeTrigger = changeTriggerTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}
		private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e) {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
				try {
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.trkText = humidity.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = humidity.DataRate;
				} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }

			} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
				try {
					dataIntervalTrk.isIntValue = true;
					if (humidity.DeviceID != DeviceID.PN_1046)
						dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.trkText = humidity.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = humidity.DataInterval;
				} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}
	}
}
